# PCACEdays
# recoding of PC-ACE files into Pajek's multirelational temporal files
# granularity is 1 day
# ------------------------------------------------------------------
# Vladimir Batagelj, 30. July 2007
# based on KEDSdays
# Vladimir Batagelj, 27. November 2004
# ------------------------------------------------------------------
# Usage:
#   PCACEdays(PCACE_file,Pajek_file)
# Examples:
#   PCACEdays('VIOLENCE.csv','violenceD.net')
# ------------------------------------------------------------------
# http://www.pc-ace.com/
# http://vlado.fmf.uni-lj.si/pub/networks/pajek/
# ------------------------------------------------------------------

PCACEdays <- function(fdat,fnet){

  get.codes <- function(line){
    nlin <<- nlin + 1;
    z <- unlist(strsplit(line,";"))
    if (length(z)==4) {
      d <- unlist(strsplit(z[1],"\\."))
      if (length(d)==3) {
        t <- as.numeric(d[1])+100*as.numeric(monthN[d[2]])+
             10000*as.numeric(d[3])
#        cat(nlin,d,t,t0,'\n')
        if (t<t0) t0 <<- t
        u <- z[2]; v <- z[4]; r <- z[3]
        if (!exists(u,env=act,inherits=FALSE)){
          nver <<- nver + 1; assign(u,nver,env=act) }
        if (!exists(v,env=act,inherits=FALSE)){
          nver <<- nver + 1; assign(v,nver,env=act) }
        if (!exists(r,env=rel,inherits=FALSE)){
          nrel <<- nrel + 1; assign(r,nrel,env=rel) }
      } else {
        cat("*** error in line ",nlin," : ",line,"\n")
      }
    } else {
      cat("*** error in line ",nlin," : ",line,"\n")
    }
  }

  recode <- function(line){
    nlin <<- nlin + 1;
    z <- unlist(strsplit(line,";"))
    if (length(z)==4) {d <- unlist(strsplit(z[1],"\\."))
      if (length(d)==3) {
        t <- as.numeric(d[1])+100*as.numeric(monthN[d[2]])+
             10000*as.numeric(d[3])
        cat(get(z[3],env=rel,inherits=FALSE),': ',
          get(z[2],env=act,inherits=FALSE),' ',
          get(z[4],env=act,inherits=FALSE),' 1 [',
          Days(t %% 100,(t %% 10000) %/% 100,1900 + t %/% 10000) - t0,
          ']\n',sep='',file=net)
      }
    }
  }

  Days <- function(d, m, y){
    p <- (0 == y %% 4) & (0 != y %% 100) | (0 == y %% 400)
    z <- y - 1; n <- z %/% 4 - z %/% 100 + z %/% 400
    n <- n + 365*y + 30*(m - 1) + d + e[m]
    if (p & (m > 2)) {n <-  n + 1}
    n
  }

  cat('PCACEdays: PC-ACE -> Pajek\n')
  ts <- strsplit(as.character(Sys.time())," ")[[1]][2]
  e <- c(1,2,0,1,1,2,2,3,4,4,5,5)
  monthN <- seq(12)
#  names(monthN) <- list("jan","feb","mar","apr","may","jun","jul","aug","sep","oct","nov","dec")
  names(monthN) <- list("jan","feb","mar","apr","maj","jun","jul","avg","sep","okt","nov","dec")
  act <- new.env(TRUE,parent = baseenv())
  rel <- new.env(TRUE,parent = baseenv())
  dat <- file(fdat,"r"); net <- file(fnet,"w")
  # lst <- file('keds.dbg',"w"); dni <- 0
  nver <- 0; nrel <- 0; t0 <- 9999999; nlin <- 0
  line <- readLines(dat,n=1)
  lines <- readLines(dat); close(dat)
  sapply(lines,get.codes)
  a <- sort(ls(envir=act)); n <- length(a)
  cat(paste('% Recoded by PCACEdays,',date()),"\n",file=net)
  cat("*vertices",n,"\n",file=net)
  for(i in 1:n){ assign(a[i],i,env=act);
    cat(i,' "',a[i],'" [1-*]\n',sep='',file=net) }
  b <- sort(ls(envir=rel)); m <- length(b)
  for(i in 1:m){ assign(a[i],i,env=act);
  cat("*arcs :",get(b[i],env=rel,inherits=FALSE),' "',
  b[i],'"\n',sep='',file=net) }
  t0 <- Days(t0 %% 100,(t0 %% 10000) %/% 100,1900 + t0 %/% 10000) - 1
  cat("*arcs\n",file=net); nlin <- 0
  sapply(lines,recode)
  cat(' ',nlin,'lines processed\n'); close(net)
  te <- strsplit(as.character(Sys.time())," ")[[1]][2]
  cat('  start:',ts,'  finish:',te,'\n')
}

#PCACEdays('VIOLENCE.csv','violenceD.net')


